@@ -11,10 +11,12 @@
 
 # Rename that back to TranspositionTable, once done.
 package Chess::Plisco::Engine::TranspositionTable;
-$Chess::Plisco::Engine::TranspositionTable::VERSION = 'v1.0.2';
+$Chess::Plisco::Engine::TranspositionTable::VERSION = 'v1.0.3';
 use strict;
 use integer;
 
+use Time::HiRes qw(gettimeofday tv_interval);
+
 # Macros from Chess::Plisco::Macro are already expanded here!
 use Chess::Plisco::Engine::Constants;
 
@@ -51,12 +53,8 @@ use constant GENERATION_CYCLE => 255 + GENERATION_DELTA;
 use constant GENERATION_MASK => (0xFF << (GENERATION_BITS)) & 0xFF;
 
 my $generation = 0;
-
-# FIXME! Inline this!
-my $relative_age = sub {
-	return (GENERATION_CYCLE + $generation - $_[0]) & GENERATION_MASK;
-};
-
+my $last_hashfull;
+my $last_hashfull_probe = 0;
 
 sub new {
 	my ($class, $size) = @_;
@@ -131,12 +129,12 @@ sub probe {
 	# Nothing found. Find a bucket to replace.
 	my ($depth, $generation) = unpack 'CC', substr $cluster, 8;
 	my $bucket_index = 0;
-	my $best = $depth - 8 * $relative_age->($generation);
+	my $best = $depth - 8 * (GENERATION_CYCLE & GENERATION_MASK);
 	for (my $i = 1; $i < @keys; ++$i) {
 		my $offset = 8 + $i * BUCKET_BYTES;
 		my $repl_bucket = substr $cluster, $offset, BUCKET_BYTES;
 		my ($repl_depth, $repl_generation) = unpack 'CC', $repl_bucket;
-		if ($repl_depth - 8 * $relative_age->($repl_generation) < $best) {
+		if ($repl_depth - 8 * ((GENERATION_CYCLE + $generation - $repl_generation) & GENERATION_MASK) < $best) {
 			$depth = $repl_depth;
 			$generation = $repl_generation;
 			$bucket_index = $i;
@@ -171,7 +169,7 @@ sub store {
 	if ($bound == BOUND_EXACT || $key != $k
 		|| ($stored_depth = unpack('C', substr $bucket, 0, 1) || 0) # Always false, forces stored_depth to be defined.
 		|| $depth - DEPTH_ENTRY_OFFSET + ($pv << 1) > $stored_depth - 4
-		|| $relative_age->($stored_depth) & GENERATION_MASK) {
+		|| ((GENERATION_CYCLE + $generation - $stored_depth) & GENERATION_MASK)) {
 		# Overwrite!
 		substr($self->[$cluster_index], $bucket_index << 1, 2) = pack 'S', $k; # Key.
 		substr($self->[$cluster_index], 8 + ($bucket_index << 3), BUCKET_BYTES) =
@@ -188,6 +186,17 @@ sub store {
 sub hashfull {
 	my ($self, $max_age) = @_;
 
+	# Do not do that more than once a second.
+	my $now = [gettimeofday];
+	if (defined $last_hashfull) {
+		no integer;
+
+		my $elapsed = tv_interval $last_hashfull_probe, $now;
+		if ($elapsed < 1) {
+			return $last_hashfull;
+		}
+	}
+
 	my $max_age_internal = $max_age << (GENERATION_BITS);
 	my $cnt = 0;
 
@@ -202,7 +211,7 @@ sub hashfull {
 			next if ($k_lo | $k_hi) == 0;   # empty
 
 			my $base = 8 + ($bi << 3);
-			my $gen  = $c[$base + 1];
+			my $gen = $c[$base + 1];
 
 			my $age =
 				(GENERATION_CYCLE + $generation - $gen)
@@ -212,7 +221,10 @@ sub hashfull {
 		}
 	}
 
-	return int($cnt / CLUSTER_CAPACITY);
+	$last_hashfull = int($cnt / CLUSTER_CAPACITY);
+	$last_hashfull_probe = $now;
+
+	return $last_hashfull;
 }
 
 1;
